from django.db import models
try:
    # django < 1.10
    from django.core.urlresolvers import reverse
except ImportError:
    # django >= 1.10
    from django.urls import reverse
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as __


@python_2_unicode_compatible
class BusinessRuleModel(models.Model):
    name = models.TextField(unique=True, verbose_name=__('name'))
    description = models.TextField(blank=True, verbose_name=__('description'))
    rule_data = models.TextField(verbose_name=__('rule data'))
    rules = models.TextField(verbose_name=__('rules'), default={})

    def get_absolute_url(self):
        return reverse('django_business_rules:business-rule-form', kwargs={'pk': self.pk})

    def __str__(self):
        return self.name



from __future__ import unicode_literals

from django.contrib import admin

from django_business_rules.models import BusinessRuleModel


@admin.register(BusinessRuleModel)
class BusinessRuleAdmin(admin.ModelAdmin):
    pass



from __future__ import unicode_literals

from django.conf.urls import url

from django_business_rules.views import BusinessRuleFormView, BusinessRuleListView


urlpatterns = [
    url(r'^business-rule/$', BusinessRuleListView.as_view(), name='business-rule-list'),
    url(r'^business-rule/(?P<pk>[0-9]+)/$', BusinessRuleFormView.as_view(), name='business-rule-form'),
]



import inspect
import pkgutil

from django.conf import settings
from django.core.management.base import BaseCommand

from django_business_rules.business_rule import BusinessRule
from django_business_rules.models import BusinessRuleModel


class BusinessRuleGenerateException(Exception):
    pass


class Command(BaseCommand):
    help = 'Updates rules data in database'
    BUSINESS_RULE_MODULE_NAME = 'rules'

    def handle(self, *args, **options):
        self.stdout.write('This command will override all rules data in database.')
        business_rule_classes = self._find_business_rule_classes(options)
        self._validate(business_rule_classes, options)
        self._save(business_rule_classes, options)
        self.stdout.write('Done with SUCCESS.')

    def _find_business_rule_classes(self, options):
        result = []
        root_path = settings.BASE_DIR
        self._debug('Looking for business rules in: {} ...'.format(root_path), options)
        for module_loader, module_name, is_pkg in pkgutil.walk_packages():
            if self._is_business_rule_module(module_loader, module_name, is_pkg, root_path):
                module = module_loader.find_module(module_name).load_module(module_name)
                result.extend(self._get_business_rule_classes(module))
        self._debug('Found business rules: {}'.format(result), options)
        return result

    @classmethod
    def _is_business_rule_module(cls, module_loader, module_name, is_pkg, root_path):
        return module_loader.path.startswith(root_path) \
               and not is_pkg \
               and module_name.endswith(Command.BUSINESS_RULE_MODULE_NAME)

    def _get_business_rule_classes(self, module):
        result = []
        for _, obj in inspect.getmembers(module):
            if self._is_business_rule_class(obj):
                result.append(obj)
        return result

    @classmethod
    def _is_business_rule_class(cls, obj):
        return inspect.isclass(obj) and issubclass(obj, BusinessRule) and obj is not BusinessRule

    def _validate(self, business_rule_classes, options):
        self._debug('Validating business rules...', options)
        unique_rule_names = {}
        for rule_class in business_rule_classes:
            name = rule_class.get_name()
            if name in unique_rule_names:
                raise BusinessRuleGenerateException(
                    'Not unique names for classes: {} and {}'.format(
                        rule_class, unique_rule_names[name]
                    )
                )
            unique_rule_names[name] = rule_class

        self._debug('Vaidation done', options)

    def _save(self, business_rule_classes, options):
        self._debug('Deleting business rules', options)
        BusinessRuleModel.objects.all().delete()
        self._debug('Saving business rules...', options)
        for rule_class in business_rule_classes:
            self._debug('Generating: {}'.format(rule_class.get_name()), options)
            rule_class.generate()
        self._debug('Saving done.', options)

    def _debug(self, text, options):
        if options['verbosity'] > 1:
            self.stdout.write(text)



from __future__ import unicode_literals

import json

from business_rules import engine, utils

from django_business_rules.models import BusinessRuleModel


class BusinessRule(object):
    name = None
    description = ''
    variables = None
    actions = None

    @classmethod
    def generate(cls):
        cls._validate_rule_data()
        BusinessRuleModel.objects.update_or_create(
            name=cls.get_name(),
            defaults=cls._get_update_values(),
        )

    @classmethod
    def _validate_rule_data(cls):
        assert cls.variables is not None
        assert cls.actions is not None

    @classmethod
    def _get_update_values(cls):
        update_values = {
            'rule_data': cls._get_rule_data(),
        }
        if cls.description:
            update_values['description'] = cls.description
        return update_values

    @classmethod
    def _get_rule_data(cls):
        rule_data = utils.export_rule_data(cls.variables, cls.actions)
        return json.dumps(rule_data)

    @classmethod
    def run_all(cls, obj, stop_on_first_trigger=True):
        return engine.run_all(
            rule_list=cls.get_rules(),
            defined_variables=cls._get_variables_instance(obj),
            defined_actions=cls._get_actions_instance(obj),
            stop_on_first_trigger=stop_on_first_trigger
        )

    @classmethod
    def _get_variables_instance(cls, obj):
        return cls.variables(obj)

    @classmethod
    def _get_actions_instance(cls, obj):
        return cls.actions(obj)

    @classmethod
    def save_rules(cls, rules):
        rules_json = json.dumps(rules)
        BusinessRuleModel.objects.filter(name=cls.get_name()).update(rules=rules_json)

    @classmethod
    def get_rules(cls):
        return cls._get_object_from_json(
            BusinessRuleModel.objects.get(name=cls.get_name()).rules
        )

    @classmethod
    def get_rule_data(cls):
        return cls._get_object_from_json(
            BusinessRuleModel.objects.get(name=cls.get_name()).rule_data
        )

    @classmethod
    def get_name(cls):
        return cls.name if cls.name else cls._get_default_name()

    @classmethod
    def _get_default_name(cls):
        return '{}.{}'.format(cls.__module__, cls.__name__)

    @classmethod
    def _get_object_from_json(cls, text):
        return json.loads(text)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-31 11:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='BusinessRuleModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.TextField(editable=False, unique=True, verbose_name='name')),
                ('description', models.TextField(blank=True, editable=False, verbose_name='description')),
                ('rule_data', models.TextField(verbose_name='rule data')),
                ('rules', models.TextField(verbose_name='rules')),
            ],
        ),
    ]



from __future__ import unicode_literals

from django.views.generic.edit import UpdateView
from django.views.generic.list import ListView

from django_business_rules.models import BusinessRuleModel


class BusinessRuleListView(ListView):
    model = BusinessRuleModel
    context_object_name = 'business_rule_list'
    template_name = 'django_business_rules/business_rule_list.html'


class BusinessRuleFormView(UpdateView):
    model = BusinessRuleModel
    fields = ('rules', )
    template_name = 'django_business_rules/business_rule_form.html'
